@@ -6,10 +6,12 @@ import re
 
 from datetime import datetime
 from enum import Enum, EnumMeta
-from typing import (Any, Callable, List, Mapping, NamedTuple, Optional, TextIO, Tuple, Union)
+from typing import (Any, Callable, List, Mapping, NamedTuple, Optional, TextIO, Tuple, Type, Union)
 from bson import ObjectId
+
 import numpy as np
 import pandas as pd
+
 from ..validation import input as input_checker
 from ..error.utils import InputDataError
 
@@ -18,6 +20,15 @@ class _MISSING_TYPE:
     def __bool__(self):
         return False
 
+    def __getattribute__(self, name: str) -> Any:
+        try:
+            return super().__getattribute__(name)
+        except AttributeError:
+            return MISSING
+
+    def __getitem__(self, name: str) -> Any:
+        return self.__getattribute__(name)
+
 
 MISSING = _MISSING_TYPE()
 
@@ -26,37 +37,89 @@ class Field:
     def __init__(self,
                  default: Union[Any, _MISSING_TYPE] = MISSING,
                  default_factory: Union[Callable, _MISSING_TYPE] = MISSING,
-                 metadata: Mapping[str, Any] = None):
+                 metadata: Mapping[str, Any] = None,
+                 tags: List[str] = []):
         if metadata is None:
             metadata = {}
         self.default = default
         self.default_factory = default_factory
         self.metadata = metadata
+        self.tags = tags
         return
 
 
 def field(default: Union[Any, _MISSING_TYPE] = MISSING,
           default_factory: Union[Callable, _MISSING_TYPE] = MISSING,
-          metadata: Mapping[str, Any] = None) -> Any:
+          metadata: Mapping[str, Any] = None,
+          tags: List[str] = []) -> Any:
     if metadata is None:
         metadata = {}
     if default is not MISSING and default_factory is not MISSING:
         raise ValueError('cannot specify both default and default_factory')
     return Field(default=default,
                  default_factory=default_factory,
-                 metadata=metadata)
+                 metadata=metadata,
+                 tags=tags)
 
 
-def is_enum_type(v):
-    return getattr(v, '__class__', None) is EnumMeta
+def is_union_type(t) -> bool:
+    return getattr(t, '__origin__', None) is Union
 
 
-def is_list_type(v):
-    return getattr(v, '__origin__', None) is List
+def union_expander(func):
+    def _union_expander(t,
+                        expand_union: bool = False,
+                        *args,
+                        **kwargs) -> Optional[Type]:
+        if expand_union and is_union_type(t):
+            t_l = t.__args__
+            rv_list = []
+            for t_c in t_l:
+                rv = func(t_c, *args, **kwargs)
+                if rv:
+                    return rv
+            return None
+        return func(t, *args, **kwargs)
+
+    return _union_expander
 
 
-def is_numeric_type(t):
-    return True if t in [int, float] else False
+@union_expander
+def is_enum_type(t) -> bool:
+    if getattr(t, '__class__', None) is EnumMeta:
+        return t
+
+
+def is_enum_value(enum_cls, v) -> bool:
+    try:
+        enum_cls(v)
+    except ValueError:
+        return False
+    return True
+
+
+def is_enum_key(enum_cls, k) -> bool:
+    return getattr(enum_cls, k, MISSING) is not MISSING
+
+
+@union_expander
+def is_list_type(t) -> Type:
+    if getattr(t, '__origin__', None) is List:
+        return t.__args__[0]
+    else:
+        return None
+
+
+@union_expander
+def is_numeric_type(t) -> Optional[Type]:
+    if t in [int, float]:
+        return True
+
+
+@union_expander
+def is_datatuple_type(t: Type) -> Optional[Type]:
+    if inspect.isclass(t) and issubclass(t, DataTuple):
+        return t
 
 
 class NumericDataRange(NamedTuple):
@@ -77,19 +140,22 @@ class DataTuple:
     __annotations__: dict = {}
     _field_data: dict = {}
 
-    def __new__(cls, init_default: bool = True, **kwargs):
+    def __new__(cls,
+                init_default: bool = True,
+                skip_init: bool = False,
+                **kwargs):
         # special annotation parse
         cls.__annotations__ = {
             k[2:-2] if k.startswith('__') and k.endswith('__') else k: v
             for k, v in cls.__annotations__.items()
         }
-        if kwargs:
-            return cls.from_dict(kwargs, init_default=init_default)
-        else:
+        if skip_init:
             return super().__new__(cls, **kwargs)
+        else:
+            return cls.from_dict(kwargs, init_default=init_default)
 
     @classmethod
-    def load_json_str(cls, s: str) -> Union[list, dict]:
+    def load_json_str(cls, s: str) -> Union[list, dict, str]:
         assert isinstance(s, str)
         try:
             s = json.loads(s)
@@ -104,7 +170,7 @@ class DataTuple:
                    attribute_value: Any) -> Any:
         value: Any
         if attribute_type is type(None):
-            if attribute_value is None:
+            if not attribute_value:
                 value = None
             else:
                 raise InputDataError(
@@ -126,11 +192,12 @@ class DataTuple:
                                                        attribute_value)
         elif attribute_type is datetime:
             time_format = self.get_attribute_field_metadata_by_key(
-                attribute_name, TIME_FORMAT_META)
+                attribute_name, TIME_FORMAT_META) or '%Y-%m-%d %H:%M:%S'
             value = input_checker.ensure_not_none_datetime(
                 attribute_name, attribute_value, time_format)
         elif is_list_type(attribute_type):
-            list_attribute_type = attribute_type.__args__[0]
+            list_attribute_type = is_list_type(attribute_type)
+            assert list_attribute_type
             if isinstance(attribute_value, str):
                 attribute_value = self.load_json_str(attribute_value)
             if not isinstance(attribute_value, list):
@@ -140,11 +207,31 @@ class DataTuple:
                 for item in attribute_value
             ]
         elif is_enum_type(attribute_type):
-            value = input_checker.ensure_not_none_enum(attribute_name,
-                                                       attribute_value,
-                                                       attribute_type)
-        elif inspect.isclass(attribute_type) and issubclass(
-                attribute_type, DataTuple):
+            value = MISSING
+            try:
+                if attribute_value is None and None in [
+                        v.value for v in attribute_type.__members__.values()
+                ]:
+                    value = attribute_type(None)
+                else:
+                    value = input_checker.ensure_not_none_enum(
+                        attribute_name, attribute_value, attribute_type)
+            except:
+                try:
+                    value = input_checker.ensure_not_none_enum(
+                        attribute_name, int(attribute_value), attribute_type)
+                except:
+                    try:
+                        value = input_checker.ensure_not_none_enum(
+                            attribute_name, str(int(attribute_value)),
+                            attribute_type)
+                    except:
+                        pass
+            if value == MISSING:
+                raise InputDataError(
+                    f'can not parse enum type={attribute_type}, attribute_value={attribute_value}'
+                )
+        elif is_datatuple_type(attribute_type):
             if isinstance(attribute_value, str):
                 attribute_value = self.load_json_str(attribute_value)
             if attribute_type is DataTuple:
@@ -162,10 +249,10 @@ class DataTuple:
                                        **attribute_value)
             elif isinstance(attribute_value, str):
                 value = attribute_type(init_default=self.init_default,
-                                       **json.loads(attribute_value))
+                                       **json.loads(attribute_value or '{}'))
             else:
                 raise InputDataError(f'{attribute_value} is not dict')
-        elif getattr(attribute_type, '__origin__', None) is Union:
+        elif is_union_type(attribute_type):
             attribute_type_list = attribute_type.__args__
             value = MISSING
             for attribute_type in attribute_type_list:
@@ -195,8 +282,7 @@ class DataTuple:
         elif attribute_type is Any:
             value = attribute_value
         else:
-            raise NotImplementedError(
-                f'datatuple does not support type={attribute_type}')
+            value = attribute_value
         # check data range if attribute type is numeric
         if is_numeric_type(attribute_type):
             data_range = self.get_attribute_field_metadata_by_key(
@@ -222,12 +308,16 @@ class DataTuple:
     def get_attribute_field_metadata_by_key(self,
                                             attribute_name: str,
                                             metadata_key: str,
+                                            default: Any = MISSING,
                                             ensure=True) -> Any:
         field_data = self.get_attribute_field_data(attribute_name)
         if not field_data:
             raise InputDataError(f'no field data {attribute_name}')
         try:
-            return field_data.metadata.get(metadata_key)
+            if default is not MISSING:
+                return field_data.metadata.get(metadata_key, default)
+            else:
+                return field_data.metadata.get(metadata_key)
         except Exception:
             if ensure:
                 raise InputDataError(
@@ -241,29 +331,24 @@ class DataTuple:
                   init_default: bool = True,
                   ignore_error=False,
                   **kwargs):
-        instance = cls()
+        instance = cls(skip_init=True)
         instance.init_default = init_default
         for attribute_name in instance.__annotations__.keys():
             instance.save_attribute_field_data(attribute_name)
+            value = MISSING
             if attribute_name in d and d[attribute_name] is not MISSING:
-                try:
-                    setattr(instance, attribute_name, d[attribute_name])
-                except:
-                    if not ignore_error:
-                        raise
+                value = d[attribute_name]
             else:
                 field_data = instance.get_attribute_field_data(attribute_name)
                 if field_data.default is not MISSING and init_default:
-                    default_value = field_data.default
+                    value = field_data.default
                 elif field_data.default_factory is not MISSING and init_default:
-                    default_value = field_data.default_factory()
-                else:
-                    continue
-                try:
-                    setattr(instance, attribute_name, default_value)
-                except:
-                    if not ignore_error:
-                        raise
+                    value = field_data.default_factory()
+            try:
+                setattr(instance, attribute_name, value)
+            except:
+                if not ignore_error:
+                    raise
         return instance
 
     def _update_by_dict(self, obj):
@@ -281,11 +366,35 @@ class DataTuple:
             raise InputDataError(f'can not update with {obj}')
         return self
 
+    def parse_json_auto(self,
+                        attribute_value: Any,
+                        parse_any_objid: bool = False) -> Any:
+        if is_enum_type(attribute_value.__class__):
+            value = attribute_value.value
+        elif isinstance(attribute_value, ObjectId) and parse_any_objid:
+            value = str(attribute_value)
+        elif isinstance(attribute_value, list):
+            value = [
+                self.parse_json_auto(v, parse_any_objid)
+                for v in attribute_value
+            ]
+        elif isinstance(attribute_value, dict):
+            value = {
+                k: self.parse_json_auto(v, parse_any_objid)
+                for k, v in attribute_value.items()
+            }
+        elif isinstance(attribute_value, DataTuple):
+            value = attribute_value.to_json()
+        else:
+            value = attribute_value
+        return value
+
     def parse_json(self,
                    attribute_name: str,
                    attribute_type: Any,
                    attribute_value: Any = MISSING,
                    recursive_func: Optional[callable] = None,
+                   parse_any_objid: bool = True,
                    **kwargs):
         value = MISSING
         recursive_func = recursive_func if recursive_func else self.parse_json
@@ -293,15 +402,18 @@ class DataTuple:
             attribute_value = getattr(self, attribute_name)
             if attribute_value is MISSING:
                 return attribute_value
-        if attribute_type in (int, float, str, bool, dict, Any, type(None)):
+        if attribute_type in (int, float, str, bool, dict, type(None)):
             value = attribute_value
+        elif attribute_type == Any:
+            value = self.parse_json_auto(attribute_value,
+                                         parse_any_objid=parse_any_objid)
         elif attribute_type is ObjectId:
             value = input_checker.ensure_not_none_objid_str(
                 attribute_name, attribute_value)
         elif attribute_type is datetime:
             attribute_value: datetime
             time_format = self.get_attribute_field_metadata_by_key(
-                attribute_name, TIME_FORMAT_META)
+                attribute_name, TIME_FORMAT_META) or '%Y-%m-%d %H:%M:%S'
             value = attribute_value.strftime(time_format)
         elif is_list_type(attribute_type):
             list_attribute_type = attribute_type.__args__[0]
@@ -312,10 +424,16 @@ class DataTuple:
                                list_attribute_type,
                                item,
                                recursive_func=recursive_func,
+                               parse_any_objid=parse_any_objid,
                                **kwargs) for item in attribute_value
             ]
         elif is_enum_type(attribute_type):
-            return attribute_value.value
+            try:
+                return attribute_value.value
+            except:
+                raise InputDataError(
+                    f'attribute_value={attribute_value} is not enum type of {attribute_type}'
+                )
         elif inspect.isclass(attribute_type) and issubclass(
                 attribute_type, DataTuple):
             if attribute_type is DataTuple:
@@ -331,12 +449,14 @@ class DataTuple:
                                                    None)
                 if recursive_func_datatuple:
                     value = attribute_value.to_dict(
-                        parser=recursive_func_datatuple, **kwargs)
+                        parser=recursive_func_datatuple,
+                        parse_any_objid=parse_any_objid,
+                        **kwargs)
                 else:
                     raise Exception(
                         f'Can not use {func_name} on datatuple {attribute_value.__class__.__name__}'
                     )
-        elif getattr(attribute_type, '__origin__', None) is Union:
+        elif is_union_type(attribute_type):
             attribute_type_list = attribute_type.__args__
             for attribute_type in attribute_type_list:
                 try:
@@ -344,28 +464,37 @@ class DataTuple:
                                            attribute_type,
                                            attribute_value,
                                            recursive_func=recursive_func,
+                                           parse_any_objid=parse_any_objid,
                                            **kwargs)
-                except InputDataError:
+                except:
                     continue
                 if value is not MISSING:
                     break
         else:
-            raise NotImplementedError(
-                f'datatuple does not support type={attribute_type}')
+            value = MISSING
         return value
 
     def to_dict(self,
                 parser: Optional[callable] = None,
+                attributes: Optional[List[str]] = None,
+                tags: Optional[List[str]] = None,
                 default: Any = MISSING,
+                parse_any_objid: bool = False,
                 **kwargs) -> dict:
         d = {}
+        if tags:
+            attributes = self.get_attribute_names(tags=tags)
         for attribute_name, attribute_type in self.__annotations__.items():
+            if attributes and attribute_name not in attributes:
+                continue
             if getattr(self, attribute_name) is not MISSING and not isinstance(
                     getattr(self, attribute_name), Field):
                 if parser:
                     attribute_value = parser(attribute_name,
                                              attribute_type,
+                                             tags=tags,
                                              default=default,
+                                             parse_any_objid=parse_any_objid,
                                              **kwargs)
                 else:
                     attribute_value = getattr(self, attribute_name)
@@ -376,14 +505,42 @@ class DataTuple:
                 d[attribute_name] = default
         return d
 
-    def to_json(self, default: Any = MISSING, **kwargs) -> dict:
-        return self.to_dict(parser=self.parse_json, default=default, **kwargs)
+    def to_json(self,
+                attributes: Optional[List[str]] = None,
+                tags: Optional[List[str]] = None,
+                default: Any = MISSING,
+                **kwargs) -> dict:
+        return self.to_dict(parser=self.parse_json,
+                            attributes=attributes,
+                            tags=tags,
+                            default=default,
+                            parse_any_objid=True,
+                            **kwargs)
+
+    def to_tuple(self,
+                 attributes: Optional[List[str]] = None,
+                 tags: Optional[List[str]] = None,
+                 default: Any = MISSING,
+                 json_to_string: bool = False,
+                 **kwargs) -> tuple:
+        j = self.to_json(attributes=attributes,
+                         tags=tags,
+                         default=default,
+                         **kwargs)
+        t_l = [(k, v) for k, v in j.items()]
+        t_l = sorted(t_l, key=lambda t: attributes.index(t[0]))
+        t = tuple(
+            json.dumps(t[1], ensure_ascii=False) if json_to_string and (
+                isinstance(t[1], list) or isinstance(t[1], dict)) else t[1]
+            for t in t_l)
+        return t
 
     def parse_bson(self,
                    attribute_name,
                    attribute_type,
                    attribute_value=MISSING,
                    recursive_func: Optional[callable] = None,
+                   parse_any_objid: bool = False,
                    **kwargs):
         recursive_func = recursive_func if recursive_func else self.parse_bson
         if attribute_value is MISSING:
@@ -395,11 +552,18 @@ class DataTuple:
                                     attribute_type,
                                     attribute_value=attribute_value,
                                     recursive_func=recursive_func,
+                                    parse_any_objid=parse_any_objid,
                                     **kwargs)
         return value
 
-    def to_bson(self, default: Any = MISSING, **kwargs) -> dict:
-        return self.to_dict(parser=self.parse_bson, default=default, **kwargs)
+    def to_bson(self,
+                attributes: Optional[str] = None,
+                default: Any = MISSING,
+                **kwargs) -> dict:
+        return self.to_dict(parser=self.parse_bson,
+                            attributes=attributes,
+                            default=default,
+                            **kwargs)
 
     def ensure_attributes(self,
                           ensure_attributes: Optional[List[str]] = None,
@@ -423,9 +587,11 @@ class DataTuple:
 
     def __setattr__(self, attribute_name: str, attribute_value: Any):
         if attribute_name in self.__annotations__:
-            attribute_type = self.__annotations__[attribute_name]
-            attribute_value = self.parse_dict(attribute_name, attribute_type,
-                                              attribute_value)
+            if attribute_value is not MISSING:
+                attribute_type = self.__annotations__[attribute_name]
+                attribute_value = self.parse_dict(attribute_name,
+                                                  attribute_type,
+                                                  attribute_value)
             super().__setattr__(attribute_name, attribute_value)
         else:
             super().__setattr__(attribute_name, attribute_value)
@@ -440,6 +606,12 @@ class DataTuple:
             else:
                 return MISSING
 
+    def __getitem__(self, name: str) -> Any:
+        return self.__getattribute__(name)
+
+    def __setitem__(self, attribute_name: str, attribute_value: Any):
+        self.__setattr__(attribute_name, attribute_value)
+
     def __eq__(self, other: Any):
         if self.__class__ == other.__class__:
             for attribute_name in self.__annotations__.keys():
@@ -450,8 +622,45 @@ class DataTuple:
         else:
             return False
 
-    def get_attribute_names(self, **kwargs) -> List[str]:
-        return list(self.__annotations__.keys())
+    @classmethod
+    def get_attribute_names(cls, tags: List[str] = [], **kwargs) -> List[str]:
+        attribute_names = []
+        if not tags:
+            return list(cls.__annotations__.keys())
+        for attribute_name, attribute_field in cls.__dict__.items():
+            if attribute_field and isinstance(
+                    attribute_field, Field) and set(tags).intersection(
+                        set(attribute_field.tags)):
+                attribute_names.append(attribute_name)
+            else:
+                continue
+        return attribute_names
+
+    def filter_attributes(self,
+                          tags: Optional[List[str]] = None,
+                          attributes: Optional[List[str]] = None,
+                          recursive: bool = True,
+                          **kwargs):
+        if attributes:
+            remain_attributes = set(attributes).intersection(
+                set(self.get_attribute_names(tags=tags)))
+        else:
+            remain_attributes = set(self.get_attribute_names(tags=tags))
+        attributes_remove = set(
+            self.get_attribute_names()).difference(remain_attributes)
+        for attr in attributes_remove:
+            self[attr] = MISSING
+        if recursive:
+            for attr in remain_attributes:
+                attr_type = self.__annotations__[attr]
+                list_element_type = is_list_type(attr_type, expand_union=True)
+                if list_element_type and self[attr]:
+                    for i, v in enumerate(self[attr]):
+                        self[attr][i].filter_attributes(tags=tags,
+                                                        recursive=recursive)
+                if is_datatuple_type(attr_type, expand_union=True) and self[attr]:
+                    self[attr].filter_attributes(tags=tags,
+                                                 recursive=recursive)
 
 
 DEFAULT_SUPPORTED_LANG = ['en', 'zh']
@@ -513,16 +722,47 @@ class MultiLangDataTuple(DataTuple):
                                      **kwargs)
         return instance
 
+    def parse_dict(self, attribute_name: str, attribute_type: Any,
+                   attribute_value: Any) -> Any:
+        if inspect.isclass(attribute_type) and \
+                issubclass(attribute_type, MultiLangData) and \
+                isinstance(attribute_value, str):
+            value = self.load_json_str(attribute_value)
+            if isinstance(value, str):
+                value = attribute_type(
+                    **{lang: attribute_value
+                       for lang in attribute_type.langs})
+                return value
+        value = super().parse_dict(attribute_name=attribute_name,
+                                   attribute_type=attribute_type,
+                                   attribute_value=attribute_value)
+        return value
+
+    @staticmethod
+    def is_multi_lang_type(attribute_type):
+        if inspect.isclass(attribute_type) and issubclass(
+                attribute_type, MultiLangData):
+            return True
+        elif getattr(attribute_type, '__origin__', None) is Union:
+            for attribute_type_child in attribute_type.__args__:
+                if inspect.isclass(attribute_type_child) and issubclass(
+                        attribute_type_child, MultiLangData):
+                    return True
+
     def to_dict(self,
                 parser: Optional[callable] = None,
+                attributes: Optional[List[str]] = None,
                 default: Any = MISSING,
                 expand_lang: bool = False,
                 **kwargs) -> dict:
-        d = super().to_dict(parser=parser, default=default, **kwargs)
+        d = super().to_dict(parser=parser,
+                            attributes=attributes,
+                            default=default,
+                            **kwargs)
         if expand_lang:
             for attribute_name, attribute_type in self.__annotations__.items():
-                if inspect.isclass(attribute_type) and issubclass(
-                        attribute_type, MultiLangData) and attribute_name in d:
+                if self.is_multi_lang_type(
+                        attribute_type) and attribute_name in d:
                     attribute_value = d[attribute_name]
                     if isinstance(attribute_value, dict):
                         for lang, item in attribute_value.items():
@@ -531,12 +771,13 @@ class MultiLangDataTuple(DataTuple):
                         d.pop(attribute_name, None)
         return d
 
-    def get_attribute_names(self,
+    @classmethod
+    def get_attribute_names(cls,
                             expand_lang: bool = False,
                             **kwargs) -> List[str]:
-        attribute_names = list(self.__annotations__.keys())
+        attribute_names = super().get_attribute_names(**kwargs)
         if expand_lang:
-            for attribute_name, attribute_type in self.__annotations__.items():
+            for attribute_name, attribute_type in cls.__annotations__.items():
                 if inspect.isclass(attribute_type) and issubclass(
                         attribute_type,
                         MultiLangData) and attribute_name in attribute_names:
@@ -554,20 +795,18 @@ def multi_lang(data_type: Any, langs: List[str] = ['en', 'zh']):
 
     class MultiLangDataGen(MultiLangData):
         def __new__(cls, **kwargs):
-            cls.data_type = data_type
-            cls.langs = langs
             cls.__annotations__ = {lang: data_type for lang in langs}
-            if kwargs:
-                return cls.from_dict(kwargs)
-            else:
-                return super().__new__(cls, **kwargs)
+            return super().__new__(cls, **kwargs)
 
         def to_dict(self,
                     parser: Optional[callable] = None,
+                    attributes: Optional[List[str]] = None,
                     default: Any = MISSING,
                     lang: str = None,
                     **kwargs) -> dict:
-            j = super().to_dict(parser=parser, default=default)
+            j = super().to_dict(parser=parser,
+                                attributes=attributes,
+                                default=default)
             if isinstance(j, dict):
                 if default is not MISSING:
                     for l in self.__annotations__.keys():
@@ -584,6 +823,8 @@ def multi_lang(data_type: Any, langs: List[str] = ['en', 'zh']):
                 raise InputDataError(
                     f'input lang is not supported, must in {self.langs}')
 
+    MultiLangDataGen.langs = langs
+    MultiLangDataGen.data_type = data_type
     return MultiLangDataGen
 
 
@@ -679,7 +920,7 @@ def datatuples_to_sheet(datatuples: List[DataTuple],
                         encoding=encoding)
         else:
             df.to_excel(excel_writer, index=False, encoding=encoding)
-        if not excel_writer:
+        if excel_writer:
             excel_writer.save()
     else:
         raise NotImplementedError()
@@ -709,4 +950,5 @@ def datatuples_read_sheet(datatuple_class: DataTuple,
                             df=df,
                             init_default=init_default,
                             ignore_error=ignore_error,
-                            **kwargs)
\ No newline at end of file
+                            **kwargs)
+
