@@ -96,7 +96,7 @@ def get_datetime(time_zone: Optional[str]=None) -> datetime:
 
 
 def get_datetime_str(time_zone: Optional[str]=None) -> str:
-    return datetime.now() if time_zone is None or len(time_zone) == '' else datetime.now(pytz.timezone(time_zone)).strftime(DATETIME_FMT_FULL)
+    return datetime.now().strftime(DATETIME_FMT_FULL) if time_zone is None or len(time_zone) == '' else datetime.now(pytz.timezone(time_zone)).strftime(DATETIME_FMT_FULL)
 
 
 def get_date(time_zone: Optional[str]=None) -> str:
