@@ -7,7 +7,7 @@ from Cryptodome.PublicKey import RSA
 
 
 # 生成密钥对
-def generate_rsa_key(secret: str='', bits: int=4096, pcks: int=8, protection: str='scryptAndAES128-CBC') -> tuple():
+def generate_rsa_key(secret: str='', bits: int=4096, pcks: int=8, protection: str='scryptAndAES128-CBC') -> tuple:
     rsa = RSA.generate(bits)
     server_private_pem = rsa.exportKey(passphrase=secret, pkcs=pcks, protection=protection)
     server_public_pem = rsa.publickey().exportKey(passphrase=secret, pkcs=pcks, protection=protection)
