@@ -4,6 +4,7 @@ import re
 import sys
 import time
 import traceback
+from colorama import init
 from typing import Any
 from logging.handlers import TimedRotatingFileHandler
 
@@ -13,6 +14,7 @@ from ..os.file import get_file_folder, get_file_name
 LOG_FORMAT = '[%(asctime)s] level: [%(levelname)s] module: [%(filename)s] func: [%(funcName)s] lineno: [%(lineno)d] msg: [%(message)s]'
 LOG_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
 
+
 class MyLoggerHandler(logging.Handler):
 
     def __init__(self, *args, **kargs):
@@ -24,6 +26,7 @@ class MyLoggerHandler(logging.Handler):
         except Exception:
             self.handleError(record)
 
+
 class MyLogLevel():
     CRITICAL = 50
     FATAL = CRITICAL
@@ -34,9 +37,10 @@ class MyLogLevel():
     DEBUG = 10
     NOTSET = 0
 
+
 class MyLogRecord(object):
   
-    def __init__(self, msg, log_level: int=MyLogLevel.INFO, fmt: str=LOG_FORMAT, log_time_fmt: str=LOG_TIME_FORMAT, **kwargs):
+    def __init__(self, msg, log_level: int = MyLogLevel.INFO, fmt: str = LOG_FORMAT, log_time_fmt: str = LOG_TIME_FORMAT, **kwargs):
 
         self.fmt = fmt
         stack = inspect.stack()[1]
@@ -51,7 +55,8 @@ class MyLogRecord(object):
 
 class MyStdout():
 
-    def __init__(self):
+    def __init__(self, autoreset: bool = True):
+        init(autoreset=autoreset)
         self.stdout_bak = sys.stdout
         self.stderr_bak = sys.stderr
         self.stdout = self
@@ -72,25 +77,27 @@ class MyStdout():
         pass
 
 
-def init_log(file_path: str, log_level: int, log_interval: int=1, log_backup: int=30, log_time_fmt: str=LOG_TIME_FORMAT, attach_handles: list=[]):
+def init_log(file_path: str, log_level: int, log_interval: int = 1, log_backup: int = 30, log_time_fmt: str = LOG_TIME_FORMAT, attach_handles: list = [], log_format: str = LOG_FORMAT):
     file_folder = get_file_folder(file_path)
     if create_dir_if_not_exist(file_folder) is True:
-
         handler_file = TimedRotatingFileHandler(filename=file_path, when="D", interval=log_interval, backupCount=log_backup)
         handler_file.suffix = "%Y-%m-%d_%H-%M.log"
         handler_file.setLevel(log_level)
         attach_handles.append(handler_file)
 
-        return logging.basicConfig(level=log_level, format=LOG_FORMAT, datefmt=log_time_fmt, handlers=attach_handles)
+        return logging.basicConfig(level=log_level, format=log_format, datefmt=log_time_fmt, handlers=attach_handles)
+
 
 def convert_msg(log_content: str):
     if len(log_content) > 0:
         return re.search(r'\[(?P<asctime>.*?)] level: \[(?P<level>.*?)] module: \[(?P<module>.*?)] func: \[(?P<func>.*?)] lineno: \[(?P<lineno>.*?)] msg: \[(?P<msg>.*)]', log_content).groupdict()
 
+
 def format_log(log_content: Any):
     func = inspect.stack()[1]
     return {'file': get_file_name(func[1]), 'func': func[3], 'lineno': func[2], 'msg': log_content}
 
+
 def _get_normalized_locals_text(locals_dict) -> str:
     var_text_list = []
     for var_name, var_value in locals_dict.items():
