@@ -724,7 +724,7 @@ def assert_not_none_tuple_of(name: str, value: Dict_In_Sequence,
 def _check_in(name: str, value: In_T, error_type: type,
               allowed_values: Container[In_T]) -> In_T:
     if value not in allowed_values:
-        raise error_type(f"{_escape(name)} is not valid.")
+        raise error_type(f"{_escape(name)} is not valid: {_escape(value)}.")
     return value
 
 
